@@ -3,17 +3,19 @@ from typing import (
     TYPE_CHECKING,
     Any,
     AsyncIterator,
+    Awaitable,
     Dict,
     Iterable,
     Iterator,
     List,
+    Literal,
     Mapping,
     NoReturn,
     Optional,
+    Sequence,
     Union,
 )
 
-from redis.compat import Literal
 from redis.crc import key_slot
 from redis.exceptions import RedisClusterException, RedisError
 from redis.typing import (
@@ -23,7 +25,9 @@ from redis.typing import (
     KeysT,
     KeyT,
     PatternT,
+    ResponseT,
 )
+from redis.utils import deprecated_function
 
 from .core import (
     ACLCommands,
@@ -31,20 +35,134 @@ from .core import (
     AsyncDataAccessCommands,
     AsyncFunctionCommands,
     AsyncManagementCommands,
+    AsyncModuleCommands,
     AsyncScriptCommands,
     DataAccessCommands,
     FunctionCommands,
+    HotkeysMetricsTypes,
     ManagementCommands,
+    ModuleCommands,
     PubSubCommands,
-    ResponseT,
     ScriptCommands,
 )
 from .helpers import list_or_args
-from .redismodules import RedisModuleCommands
+from .redismodules import AsyncRedisModuleCommands, RedisModuleCommands
 
 if TYPE_CHECKING:
     from redis.asyncio.cluster import TargetNodesT
 
+# Not complete, but covers the major ones
+# https://redis.io/commands
+READ_COMMANDS = frozenset(
+    [
+        # Bit Operations
+        "BITCOUNT",
+        "BITFIELD_RO",
+        "BITPOS",
+        # Scripting
+        "EVAL_RO",
+        "EVALSHA_RO",
+        "FCALL_RO",
+        # Key Operations
+        "DBSIZE",
+        "DIGEST",
+        "DUMP",
+        "EXISTS",
+        "EXPIRETIME",
+        "PEXPIRETIME",
+        "KEYS",
+        "SCAN",
+        "PTTL",
+        "RANDOMKEY",
+        "TTL",
+        "TYPE",
+        # String Operations
+        "GET",
+        "GETBIT",
+        "GETRANGE",
+        "MGET",
+        "STRLEN",
+        "LCS",
+        # Geo Operations
+        "GEODIST",
+        "GEOHASH",
+        "GEOPOS",
+        "GEOSEARCH",
+        # Hash Operations
+        "HEXISTS",
+        "HGET",
+        "HGETALL",
+        "HKEYS",
+        "HLEN",
+        "HMGET",
+        "HSTRLEN",
+        "HVALS",
+        "HRANDFIELD",
+        "HEXPIRETIME",
+        "HPEXPIRETIME",
+        "HTTL",
+        "HPTTL",
+        "HSCAN",
+        # List Operations
+        "LINDEX",
+        "LPOS",
+        "LLEN",
+        "LRANGE",
+        # Set Operations
+        "SCARD",
+        "SDIFF",
+        "SINTER",
+        "SINTERCARD",
+        "SISMEMBER",
+        "SMISMEMBER",
+        "SMEMBERS",
+        "SRANDMEMBER",
+        "SUNION",
+        "SSCAN",
+        # Sorted Set Operations
+        "ZCARD",
+        "ZCOUNT",
+        "ZDIFF",
+        "ZINTER",
+        "ZINTERCARD",
+        "ZLEXCOUNT",
+        "ZMSCORE",
+        "ZRANDMEMBER",
+        "ZRANGE",
+        "ZRANGEBYLEX",
+        "ZRANGEBYSCORE",
+        "ZRANK",
+        "ZREVRANGE",
+        "ZREVRANGEBYLEX",
+        "ZREVRANGEBYSCORE",
+        "ZREVRANK",
+        "ZSCAN",
+        "ZSCORE",
+        "ZUNION",
+        # Stream Operations
+        "XLEN",
+        "XPENDING",
+        "XRANGE",
+        "XREAD",
+        "XREVRANGE",
+        # JSON Module
+        "JSON.ARRINDEX",
+        "JSON.ARRLEN",
+        "JSON.GET",
+        "JSON.MGET",
+        "JSON.OBJKEYS",
+        "JSON.OBJLEN",
+        "JSON.RESP",
+        "JSON.STRLEN",
+        "JSON.TYPE",
+        # RediSearch Module
+        "FT.EXPLAIN",
+        "FT.INFO",
+        "FT.PROFILE",
+        "FT.SEARCH",
+    ]
+)
+
 
 class ClusterMultiKeyCommands(ClusterCommandsProtocol):
     """
@@ -52,19 +170,58 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
     """
 
     def _partition_keys_by_slot(self, keys: Iterable[KeyT]) -> Dict[int, List[KeyT]]:
-        """
-        Split keys into a dictionary that maps a slot to
-        a list of keys.
-        """
+        """Split keys into a dictionary that maps a slot to a list of keys."""
+
         slots_to_keys = {}
         for key in keys:
-            k = self.encoder.encode(key)
-            slot = key_slot(k)
+            slot = key_slot(self.encoder.encode(key))
             slots_to_keys.setdefault(slot, []).append(key)
 
         return slots_to_keys
 
-    def mget_nonatomic(self, keys: KeysT, *args) -> List[Optional[Any]]:
+    def _partition_pairs_by_slot(
+        self, mapping: Mapping[AnyKeyT, EncodableT]
+    ) -> Dict[int, List[EncodableT]]:
+        """Split pairs into a dictionary that maps a slot to a list of pairs."""
+
+        slots_to_pairs = {}
+        for pair in mapping.items():
+            slot = key_slot(self.encoder.encode(pair[0]))
+            slots_to_pairs.setdefault(slot, []).extend(pair)
+
+        return slots_to_pairs
+
+    def _execute_pipeline_by_slot(
+        self, command: str, slots_to_args: Mapping[int, Iterable[EncodableT]]
+    ) -> List[Any]:
+        read_from_replicas = self.read_from_replicas and command in READ_COMMANDS
+        pipe = self.pipeline()
+        [
+            pipe.execute_command(
+                command,
+                *slot_args,
+                target_nodes=[
+                    self.nodes_manager.get_node_from_slot(slot, read_from_replicas)
+                ],
+            )
+            for slot, slot_args in slots_to_args.items()
+        ]
+        return pipe.execute()
+
+    def _reorder_keys_by_command(
+        self,
+        keys: Iterable[KeyT],
+        slots_to_args: Mapping[int, Iterable[EncodableT]],
+        responses: Iterable[Any],
+    ) -> List[Any]:
+        results = {
+            k: v
+            for slot_values, response in zip(slots_to_args.values(), responses)
+            for k, v in zip(slot_values, response)
+        }
+        return [results[key] for key in keys]
+
+    def mget_nonatomic(self, keys: KeysT, *args: KeyT) -> List[Optional[Any]]:
         """
         Splits the keys into different slots and then calls MGET
         for the keys of every slot. This operation will not be atomic
@@ -75,30 +232,17 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
         For more information see https://redis.io/commands/mget
         """
 
-        from redis.client import EMPTY_RESPONSE
-
-        options = {}
-        if not args:
-            options[EMPTY_RESPONSE] = []
-
         # Concatenate all keys into a list
         keys = list_or_args(keys, args)
+
         # Split keys into slots
         slots_to_keys = self._partition_keys_by_slot(keys)
 
-        # Call MGET for every slot and concatenate
-        # the results
-        # We must make sure that the keys are returned in order
-        all_results = {}
-        for slot_keys in slots_to_keys.values():
-            slot_values = self.execute_command("MGET", *slot_keys, **options)
+        # Execute commands using a pipeline
+        res = self._execute_pipeline_by_slot("MGET", slots_to_keys)
 
-            slot_results = dict(zip(slot_keys, slot_values))
-            all_results.update(slot_results)
-
-        # Sort the results
-        vals_in_order = [all_results[key] for key in keys]
-        return vals_in_order
+        # Reorder keys in the order the user provided & return
+        return self._reorder_keys_by_command(keys, slots_to_keys, res)
 
     def mset_nonatomic(self, mapping: Mapping[AnyKeyT, EncodableT]) -> List[bool]:
         """
@@ -114,35 +258,22 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
         """
 
         # Partition the keys by slot
-        slots_to_pairs = {}
-        for pair in mapping.items():
-            # encode the key
-            k = self.encoder.encode(pair[0])
-            slot = key_slot(k)
-            slots_to_pairs.setdefault(slot, []).extend(pair)
+        slots_to_pairs = self._partition_pairs_by_slot(mapping)
 
-        # Call MSET for every slot and concatenate
-        # the results (one result per slot)
-        res = []
-        for pairs in slots_to_pairs.values():
-            res.append(self.execute_command("MSET", *pairs))
-
-        return res
+        # Execute commands using a pipeline & return list of replies
+        return self._execute_pipeline_by_slot("MSET", slots_to_pairs)
 
     def _split_command_across_slots(self, command: str, *keys: KeyT) -> int:
         """
         Runs the given command once for the keys
         of each slot. Returns the sum of the return values.
         """
+
         # Partition the keys by slot
         slots_to_keys = self._partition_keys_by_slot(keys)
 
         # Sum up the reply from each command
-        total = 0
-        for slot_keys in slots_to_keys.values():
-            total += self.execute_command(command, *slot_keys)
-
-        return total
+        return sum(self._execute_pipeline_by_slot(command, slots_to_keys))
 
     def exists(self, *keys: KeyT) -> ResponseT:
         """
@@ -160,7 +291,7 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
         The keys are first split up into slots
         and then an DEL command is sent for every slot
 
-        Non-existant keys are ignored.
+        Non-existent keys are ignored.
         Returns the number of keys that were deleted.
 
         For more information see https://redis.io/commands/del
@@ -175,7 +306,7 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
         The keys are first split up into slots
         and then an TOUCH command is sent for every slot
 
-        Non-existant keys are ignored.
+        Non-existent keys are ignored.
         Returns the number of keys that were touched.
 
         For more information see https://redis.io/commands/touch
@@ -189,7 +320,7 @@ class ClusterMultiKeyCommands(ClusterCommandsProtocol):
         The keys are first split up into slots
         and then an TOUCH command is sent for every slot
 
-        Non-existant keys are ignored.
+        Non-existent keys are ignored.
         Returns the number of keys that were unlinked.
 
         For more information see https://redis.io/commands/unlink
@@ -202,7 +333,7 @@ class AsyncClusterMultiKeyCommands(ClusterMultiKeyCommands):
     A class containing commands that handle more than one key
     """
 
-    async def mget_nonatomic(self, keys: KeysT, *args) -> List[Optional[Any]]:
+    async def mget_nonatomic(self, keys: KeysT, *args: KeyT) -> List[Optional[Any]]:
         """
         Splits the keys into different slots and then calls MGET
         for the keys of every slot. This operation will not be atomic
@@ -213,36 +344,17 @@ class AsyncClusterMultiKeyCommands(ClusterMultiKeyCommands):
         For more information see https://redis.io/commands/mget
         """
 
-        from redis.client import EMPTY_RESPONSE
-
-        options = {}
-        if not args:
-            options[EMPTY_RESPONSE] = []
-
         # Concatenate all keys into a list
         keys = list_or_args(keys, args)
+
         # Split keys into slots
         slots_to_keys = self._partition_keys_by_slot(keys)
 
-        # Call MGET for every slot and concatenate
-        # the results
-        # We must make sure that the keys are returned in order
-        all_values = await asyncio.gather(
-            *(
-                asyncio.ensure_future(
-                    self.execute_command("MGET", *slot_keys, **options)
-                )
-                for slot_keys in slots_to_keys.values()
-            )
-        )
+        # Execute commands using a pipeline
+        res = await self._execute_pipeline_by_slot("MGET", slots_to_keys)
 
-        all_results = {}
-        for slot_keys, slot_values in zip(slots_to_keys.values(), all_values):
-            all_results.update(dict(zip(slot_keys, slot_values)))
-
-        # Sort the results
-        vals_in_order = [all_results[key] for key in keys]
-        return vals_in_order
+        # Reorder keys in the order the user provided & return
+        return self._reorder_keys_by_command(keys, slots_to_keys, res)
 
     async def mset_nonatomic(self, mapping: Mapping[AnyKeyT, EncodableT]) -> List[bool]:
         """
@@ -258,39 +370,41 @@ class AsyncClusterMultiKeyCommands(ClusterMultiKeyCommands):
         """
 
         # Partition the keys by slot
-        slots_to_pairs = {}
-        for pair in mapping.items():
-            # encode the key
-            k = self.encoder.encode(pair[0])
-            slot = key_slot(k)
-            slots_to_pairs.setdefault(slot, []).extend(pair)
+        slots_to_pairs = self._partition_pairs_by_slot(mapping)
 
-        # Call MSET for every slot and concatenate
-        # the results (one result per slot)
-        return await asyncio.gather(
-            *(
-                asyncio.ensure_future(self.execute_command("MSET", *pairs))
-                for pairs in slots_to_pairs.values()
-            )
-        )
+        # Execute commands using a pipeline & return list of replies
+        return await self._execute_pipeline_by_slot("MSET", slots_to_pairs)
 
     async def _split_command_across_slots(self, command: str, *keys: KeyT) -> int:
         """
         Runs the given command once for the keys
         of each slot. Returns the sum of the return values.
         """
+
         # Partition the keys by slot
         slots_to_keys = self._partition_keys_by_slot(keys)
 
         # Sum up the reply from each command
-        return sum(
-            await asyncio.gather(
-                *(
-                    asyncio.ensure_future(self.execute_command(command, *slot_keys))
-                    for slot_keys in slots_to_keys.values()
-                )
+        return sum(await self._execute_pipeline_by_slot(command, slots_to_keys))
+
+    async def _execute_pipeline_by_slot(
+        self, command: str, slots_to_args: Mapping[int, Iterable[EncodableT]]
+    ) -> List[Any]:
+        if self._initialize:
+            await self.initialize()
+        read_from_replicas = self.read_from_replicas and command in READ_COMMANDS
+        pipe = self.pipeline()
+        [
+            pipe.execute_command(
+                command,
+                *slot_args,
+                target_nodes=[
+                    self.nodes_manager.get_node_from_slot(slot, read_from_replicas)
+                ],
             )
-        )
+            for slot, slot_args in slots_to_args.items()
+        ]
+        return await pipe.execute()
 
 
 class ClusterManagementCommands(ManagementCommands):
@@ -547,7 +661,7 @@ class ClusterManagementCommands(ManagementCommands):
                 "CLUSTER SETSLOT", slot_id, state, node_id, target_nodes=target_node
             )
         elif state.upper() == "STABLE":
-            raise RedisError('For "stable" state please use ' "cluster_setslot_stable")
+            raise RedisError('For "stable" state please use cluster_setslot_stable')
         else:
             raise RedisError(f"Invalid slot state: {state}")
 
@@ -589,6 +703,14 @@ class ClusterManagementCommands(ManagementCommands):
         """
         return self.execute_command("CLUSTER SHARDS", target_nodes=target_nodes)
 
+    def cluster_myshardid(self, target_nodes=None):
+        """
+        Returns the shard ID of the node.
+
+        For more information see https://redis.io/commands/cluster-myshardid/
+        """
+        return self.execute_command("CLUSTER MYSHARDID", target_nodes=target_nodes)
+
     def cluster_links(self, target_node: "TargetNodesT") -> ResponseT:
         """
         Each node in a Redis Cluster maintains a pair of long-lived TCP link with each
@@ -601,6 +723,16 @@ class ClusterManagementCommands(ManagementCommands):
         """
         return self.execute_command("CLUSTER LINKS", target_nodes=target_node)
 
+    def cluster_flushslots(self, target_nodes: Optional["TargetNodesT"] = None) -> None:
+        raise NotImplementedError(
+            "CLUSTER FLUSHSLOTS is intentionally not implemented in the client."
+        )
+
+    def cluster_bumpepoch(self, target_nodes: Optional["TargetNodesT"] = None) -> None:
+        raise NotImplementedError(
+            "CLUSTER BUMPEPOCH is intentionally not implemented in the client."
+        )
+
     def readonly(self, target_nodes: Optional["TargetNodesT"] = None) -> ResponseT:
         """
         Enables read queries.
@@ -627,6 +759,124 @@ class ClusterManagementCommands(ManagementCommands):
         self.read_from_replicas = False
         return self.execute_command("READWRITE", target_nodes=target_nodes)
 
+    @deprecated_function(
+        version="7.2.0",
+        reason="Use client-side caching feature instead.",
+    )
+    def client_tracking_on(
+        self,
+        clientid: Optional[int] = None,
+        prefix: Sequence[KeyT] = [],
+        bcast: bool = False,
+        optin: bool = False,
+        optout: bool = False,
+        noloop: bool = False,
+        target_nodes: Optional["TargetNodesT"] = "all",
+    ) -> ResponseT:
+        """
+        Enables the tracking feature of the Redis server, that is used
+        for server assisted client side caching.
+
+        When clientid is provided - in target_nodes only the node that owns the
+        connection with this id should be provided.
+        When clientid is not provided - target_nodes can be any node.
+
+        For more information see https://redis.io/commands/client-tracking
+        """
+        return self.client_tracking(
+            True,
+            clientid,
+            prefix,
+            bcast,
+            optin,
+            optout,
+            noloop,
+            target_nodes=target_nodes,
+        )
+
+    @deprecated_function(
+        version="7.2.0",
+        reason="Use client-side caching feature instead.",
+    )
+    def client_tracking_off(
+        self,
+        clientid: Optional[int] = None,
+        prefix: Sequence[KeyT] = [],
+        bcast: bool = False,
+        optin: bool = False,
+        optout: bool = False,
+        noloop: bool = False,
+        target_nodes: Optional["TargetNodesT"] = "all",
+    ) -> ResponseT:
+        """
+        Disables the tracking feature of the Redis server, that is used
+        for server assisted client side caching.
+
+        When clientid is provided - in target_nodes only the node that owns the
+        connection with this id should be provided.
+        When clientid is not provided - target_nodes can be any node.
+
+        For more information see https://redis.io/commands/client-tracking
+        """
+        return self.client_tracking(
+            False,
+            clientid,
+            prefix,
+            bcast,
+            optin,
+            optout,
+            noloop,
+            target_nodes=target_nodes,
+        )
+
+    def hotkeys_start(
+        self,
+        metrics: List[HotkeysMetricsTypes],
+        count: Optional[int] = None,
+        duration: Optional[int] = None,
+        sample_ratio: Optional[int] = None,
+        slots: Optional[List[int]] = None,
+        **kwargs,
+    ) -> Union[str, bytes]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-start
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    def hotkeys_stop(self, **kwargs) -> Union[str, bytes]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-stop
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    def hotkeys_reset(self, **kwargs) -> Union[str, bytes]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-reset
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    def hotkeys_get(self, **kwargs) -> list[dict[Union[str, bytes], Any]]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-get
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
 
 class AsyncClusterManagementCommands(
     ClusterManagementCommands, AsyncManagementCommands
@@ -649,11 +899,131 @@ class AsyncClusterManagementCommands(
         """
         return await asyncio.gather(
             *(
-                asyncio.ensure_future(self.execute_command("CLUSTER DELSLOTS", slot))
+                asyncio.create_task(self.execute_command("CLUSTER DELSLOTS", slot))
                 for slot in slots
             )
         )
 
+    @deprecated_function(
+        version="7.2.0",
+        reason="Use client-side caching feature instead.",
+    )
+    async def client_tracking_on(
+        self,
+        clientid: Optional[int] = None,
+        prefix: Sequence[KeyT] = [],
+        bcast: bool = False,
+        optin: bool = False,
+        optout: bool = False,
+        noloop: bool = False,
+        target_nodes: Optional["TargetNodesT"] = "all",
+    ) -> ResponseT:
+        """
+        Enables the tracking feature of the Redis server, that is used
+        for server assisted client side caching.
+
+        When clientid is provided - in target_nodes only the node that owns the
+        connection with this id should be provided.
+        When clientid is not provided - target_nodes can be any node.
+
+        For more information see https://redis.io/commands/client-tracking
+        """
+        return await self.client_tracking(
+            True,
+            clientid,
+            prefix,
+            bcast,
+            optin,
+            optout,
+            noloop,
+            target_nodes=target_nodes,
+        )
+
+    @deprecated_function(
+        version="7.2.0",
+        reason="Use client-side caching feature instead.",
+    )
+    async def client_tracking_off(
+        self,
+        clientid: Optional[int] = None,
+        prefix: Sequence[KeyT] = [],
+        bcast: bool = False,
+        optin: bool = False,
+        optout: bool = False,
+        noloop: bool = False,
+        target_nodes: Optional["TargetNodesT"] = "all",
+    ) -> ResponseT:
+        """
+        Disables the tracking feature of the Redis server, that is used
+        for server assisted client side caching.
+
+        When clientid is provided - in target_nodes only the node that owns the
+        connection with this id should be provided.
+        When clientid is not provided - target_nodes can be any node.
+
+        For more information see https://redis.io/commands/client-tracking
+        """
+        return await self.client_tracking(
+            False,
+            clientid,
+            prefix,
+            bcast,
+            optin,
+            optout,
+            noloop,
+            target_nodes=target_nodes,
+        )
+
+    async def hotkeys_start(
+        self,
+        metrics: List[HotkeysMetricsTypes],
+        count: Optional[int] = None,
+        duration: Optional[int] = None,
+        sample_ratio: Optional[int] = None,
+        slots: Optional[List[int]] = None,
+        **kwargs,
+    ) -> Awaitable[Union[str, bytes]]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-start
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    async def hotkeys_stop(self, **kwargs) -> Awaitable[Union[str, bytes]]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-stop
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    async def hotkeys_reset(self, **kwargs) -> Awaitable[Union[str, bytes]]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-reset
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
+    async def hotkeys_get(
+        self, **kwargs
+    ) -> Awaitable[list[dict[Union[str, bytes], Any]]]:
+        """
+        Cluster client does not support hotkeys command. Please use the non-cluster client.
+
+        For more information see https://redis.io/commands/hotkeys-get
+        """
+        raise NotImplementedError(
+            "HOTKEYS commands are not supported in cluster mode. Please use the non-cluster client."
+        )
+
 
 class ClusterDataAccessCommands(DataAccessCommands):
     """
@@ -802,6 +1172,7 @@ class RedisClusterCommands(
     ClusterDataAccessCommands,
     ScriptCommands,
     FunctionCommands,
+    ModuleCommands,
     RedisModuleCommands,
 ):
     """
@@ -831,6 +1202,8 @@ class AsyncRedisClusterCommands(
     AsyncClusterDataAccessCommands,
     AsyncScriptCommands,
     AsyncFunctionCommands,
+    AsyncModuleCommands,
+    AsyncRedisModuleCommands,
 ):
     """
     A class for all Redis Cluster commands
