@@ -1,13 +1,15 @@
 import asyncio
 import re
+from unittest import mock
 
 import pytest
 import pytest_asyncio
 import redis.asyncio as redis
 from redis.asyncio.connection import Connection, to_bool
+from redis.auth.token import TokenInterface
 from tests.conftest import skip_if_redis_enterprise, skip_if_server_version_lt
 
-from .compat import aclosing, mock
+from .compat import aclosing
 from .conftest import asynccontextmanager
 from .test_pubsub import wait_for_message
 
@@ -28,8 +30,8 @@ class TestRedisAutoReleaseConnectionPool:
     @staticmethod
     async def create_two_conn(r: redis.Redis):
         if not r.single_connection_client:  # Single already initialized connection
-            r.connection = await r.connection_pool.get_connection("_")
-        return await r.connection_pool.get_connection("_")
+            r.connection = await r.connection_pool.get_connection()
+        return await r.connection_pool.get_connection()
 
     @staticmethod
     def has_no_connected_connections(pool: redis.ConnectionPool):
@@ -93,19 +95,33 @@ class DummyConnection(Connection):
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
+        self._connected = False
 
     def repr_pieces(self):
         return [("id", id(self)), ("kwargs", self.kwargs)]
 
     async def connect(self):
-        pass
+        self._connected = True
 
     async def disconnect(self):
-        pass
+        self._connected = False
+
+    @property
+    def is_connected(self):
+        return self._connected
 
     async def can_read_destructive(self, timeout: float = 0):
         return False
 
+    def set_re_auth_token(self, token: TokenInterface):
+        pass
+
+    async def re_auth(self):
+        pass
+
+    def should_reconnect(self):
+        return False
+
 
 class TestConnectionPool:
     @asynccontextmanager
@@ -131,7 +147,7 @@ class TestConnectionPool:
         async with self.get_pool(
             connection_kwargs=connection_kwargs, connection_class=DummyConnection
         ) as pool:
-            connection = await pool.get_connection("_")
+            connection = await pool.get_connection()
             assert isinstance(connection, DummyConnection)
             assert connection.kwargs == connection_kwargs
 
@@ -148,8 +164,8 @@ class TestConnectionPool:
     async def test_multiple_connections(self, master_host):
         connection_kwargs = {"host": master_host[0]}
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            c1 = await pool.get_connection("_")
-            c2 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
+            c2 = await pool.get_connection()
             assert c1 != c2
 
     async def test_max_connections(self, master_host):
@@ -157,17 +173,17 @@ class TestConnectionPool:
         async with self.get_pool(
             max_connections=2, connection_kwargs=connection_kwargs
         ) as pool:
-            await pool.get_connection("_")
-            await pool.get_connection("_")
+            await pool.get_connection()
+            await pool.get_connection()
             with pytest.raises(redis.ConnectionError):
-                await pool.get_connection("_")
+                await pool.get_connection()
 
     async def test_reuse_previously_released_connection(self, master_host):
         connection_kwargs = {"host": master_host[0]}
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            c1 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
             await pool.release(c1)
-            c2 = await pool.get_connection("_")
+            c2 = await pool.get_connection()
             assert c1 == c2
 
     async def test_repr_contains_db_info_tcp(self):
@@ -180,11 +196,8 @@ class TestConnectionPool:
         async with self.get_pool(
             connection_kwargs=connection_kwargs, connection_class=redis.Connection
         ) as pool:
-            expected = (
-                "ConnectionPool<Connection<"
-                "host=localhost,port=6379,db=1,client_name=test-client>>"
-            )
-            assert repr(pool) == expected
+            expected = "host=localhost,port=6379,db=1,client_name=test-client"
+            assert expected in repr(pool)
 
     async def test_repr_contains_db_info_unix(self):
         connection_kwargs = {"path": "/abc", "db": 1, "client_name": "test-client"}
@@ -192,11 +205,97 @@ class TestConnectionPool:
             connection_kwargs=connection_kwargs,
             connection_class=redis.UnixDomainSocketConnection,
         ) as pool:
-            expected = (
-                "ConnectionPool<UnixDomainSocketConnection<"
-                "path=/abc,db=1,client_name=test-client>>"
+            expected = "path=/abc,db=1,client_name=test-client"
+            assert expected in repr(pool)
+
+    async def test_pool_disconnect(self, master_host):
+        connection_kwargs = {
+            "host": master_host[0],
+            "port": master_host[1],
+        }
+        async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
+            conn = await pool.get_connection()
+            await pool.disconnect(inuse_connections=True)
+            assert not conn.is_connected
+
+            await conn.connect()
+            await pool.disconnect(inuse_connections=False)
+            assert conn.is_connected
+
+    async def test_lock_not_held_during_connection_establishment(self):
+        """
+        Test that the connection pool lock is not held during the
+        ensure_connection call, which involves socket connection and handshake.
+        This is important for performance under high load.
+        """
+        lock_states = []
+
+        class SlowConnectConnection(DummyConnection):
+            """Connection that simulates slow connection establishment"""
+
+            async def connect(self):
+                # Check if the pool's lock is held during connection
+                # We access the pool through the outer scope
+                lock_states.append(pool._lock.locked())
+                # Simulate slow connection
+                await asyncio.sleep(0.01)
+                self._connected = True
+
+        async with self.get_pool(connection_class=SlowConnectConnection) as pool:
+            # Get a connection - this should call connect() outside the lock
+            connection = await pool.get_connection()
+
+            # Verify the lock was NOT held during connect
+            assert len(lock_states) > 0, "connect() should have been called"
+            assert lock_states[0] is False, (
+                "Lock should not be held during connection establishment"
             )
-            assert repr(pool) == expected
+
+            await pool.release(connection)
+
+    async def test_concurrent_connection_acquisition_performance(self):
+        """
+        Test that multiple concurrent connection acquisitions don't block
+        each other during connection establishment.
+        """
+        connection_delay = 0.05
+        num_connections = 3
+
+        class SlowConnectConnection(DummyConnection):
+            """Connection that simulates slow connection establishment"""
+
+            async def connect(self):
+                # Simulate slow connection (e.g., network latency, TLS handshake)
+                await asyncio.sleep(connection_delay)
+                self._connected = True
+
+        async with self.get_pool(
+            connection_class=SlowConnectConnection, max_connections=10
+        ) as pool:
+            # Start acquiring multiple connections concurrently
+            start_time = asyncio.get_running_loop().time()
+
+            # Try to get connections concurrently
+            connections = await asyncio.gather(
+                *[pool.get_connection() for _ in range(num_connections)]
+            )
+
+            elapsed_time = asyncio.get_running_loop().time() - start_time
+
+            # With proper lock handling, these should complete mostly in parallel
+            # If the lock was held during connect(), it would take num_connections * connection_delay
+            # With lock only during pop, it should take ~connection_delay (connections in parallel)
+            # We allow 2.5x overhead for system variance
+            max_allowed_time = connection_delay * 2.5
+            assert elapsed_time < max_allowed_time, (
+                f"Concurrent connections took {elapsed_time:.3f}s, "
+                f"expected < {max_allowed_time:.3f}s. "
+                f"This suggests lock was held during connection establishment."
+            )
+
+            # Clean up
+            for conn in connections:
+                await pool.release(conn)
 
 
 class TestBlockingConnectionPool:
@@ -222,12 +321,11 @@ class TestBlockingConnectionPool:
             "port": master_host[1],
         }
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            connection = await pool.get_connection("_")
+            connection = await pool.get_connection()
             assert isinstance(connection, DummyConnection)
             assert connection.kwargs == connection_kwargs
 
-    async def test_disconnect(self, master_host):
-        """A regression test for #1047"""
+    async def test_pool_disconnect(self, master_host):
         connection_kwargs = {
             "foo": "bar",
             "biz": "baz",
@@ -235,14 +333,19 @@ class TestBlockingConnectionPool:
             "port": master_host[1],
         }
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            await pool.get_connection("_")
+            conn = await pool.get_connection()
             await pool.disconnect()
+            assert not conn.is_connected
+
+            await conn.connect()
+            await pool.disconnect(inuse_connections=False)
+            assert conn.is_connected
 
     async def test_multiple_connections(self, master_host):
         connection_kwargs = {"host": master_host[0], "port": master_host[1]}
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            c1 = await pool.get_connection("_")
-            c2 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
+            c2 = await pool.get_connection()
             assert c1 != c2
 
     async def test_connection_pool_blocks_until_timeout(self, master_host):
@@ -251,11 +354,11 @@ class TestBlockingConnectionPool:
         async with self.get_pool(
             max_connections=1, timeout=0.1, connection_kwargs=connection_kwargs
         ) as pool:
-            c1 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
 
             start = asyncio.get_running_loop().time()
             with pytest.raises(redis.ConnectionError):
-                await pool.get_connection("_")
+                await pool.get_connection()
 
             # we should have waited at least some period of time
             assert asyncio.get_running_loop().time() - start >= 0.05
@@ -270,46 +373,41 @@ class TestBlockingConnectionPool:
         async with self.get_pool(
             max_connections=1, timeout=2, connection_kwargs=connection_kwargs
         ) as pool:
-            c1 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
 
             async def target():
                 await asyncio.sleep(0.1)
                 await pool.release(c1)
 
             start = asyncio.get_running_loop().time()
-            await asyncio.gather(target(), pool.get_connection("_"))
+            await asyncio.gather(target(), pool.get_connection())
             stop = asyncio.get_running_loop().time()
             assert (stop - start) <= 0.2
 
     async def test_reuse_previously_released_connection(self, master_host):
         connection_kwargs = {"host": master_host[0]}
         async with self.get_pool(connection_kwargs=connection_kwargs) as pool:
-            c1 = await pool.get_connection("_")
+            c1 = await pool.get_connection()
             await pool.release(c1)
-            c2 = await pool.get_connection("_")
+            c2 = await pool.get_connection()
             assert c1 == c2
 
     def test_repr_contains_db_info_tcp(self):
         pool = redis.ConnectionPool(
             host="localhost", port=6379, client_name="test-client"
         )
-        expected = (
-            "ConnectionPool<Connection<"
-            "host=localhost,port=6379,db=0,client_name=test-client>>"
-        )
-        assert repr(pool) == expected
+        expected = "host=localhost,port=6379,client_name=test-client"
+        assert expected in repr(pool)
 
     def test_repr_contains_db_info_unix(self):
         pool = redis.ConnectionPool(
             connection_class=redis.UnixDomainSocketConnection,
             path="abc",
+            db=0,
             client_name="test-client",
         )
-        expected = (
-            "ConnectionPool<UnixDomainSocketConnection<"
-            "path=abc,db=0,client_name=test-client>>"
-        )
-        assert repr(pool) == expected
+        expected = "path=abc,db=0,client_name=test-client"
+        assert expected in repr(pool)
 
 
 class TestConnectionPoolURLParsing:
@@ -557,23 +655,23 @@ class TestSSLConnectionURLParsing:
         import ssl
 
         class DummyConnectionPool(redis.ConnectionPool):
-            def get_connection(self, *args, **kwargs):
+            def get_connection(self):
                 return self.make_connection()
 
         pool = DummyConnectionPool.from_url("rediss://?ssl_cert_reqs=none")
-        assert pool.get_connection("_").cert_reqs == ssl.CERT_NONE
+        assert pool.get_connection().cert_reqs == ssl.CERT_NONE
 
         pool = DummyConnectionPool.from_url("rediss://?ssl_cert_reqs=optional")
-        assert pool.get_connection("_").cert_reqs == ssl.CERT_OPTIONAL
+        assert pool.get_connection().cert_reqs == ssl.CERT_OPTIONAL
 
         pool = DummyConnectionPool.from_url("rediss://?ssl_cert_reqs=required")
-        assert pool.get_connection("_").cert_reqs == ssl.CERT_REQUIRED
+        assert pool.get_connection().cert_reqs == ssl.CERT_REQUIRED
 
         pool = DummyConnectionPool.from_url("rediss://?ssl_check_hostname=False")
-        assert pool.get_connection("_").check_hostname is False
+        assert pool.get_connection().check_hostname is False
 
         pool = DummyConnectionPool.from_url("rediss://?ssl_check_hostname=True")
-        assert pool.get_connection("_").check_hostname is True
+        assert pool.get_connection().check_hostname is True
 
 
 class TestConnection:
@@ -619,9 +717,9 @@ class TestConnection:
                 "DEBUG", "ERROR", "LOADING fake message"
             )
         pool = r.connection_pool
-        assert not pipe.connection
-        assert len(pool._available_connections) == 1
-        assert not pool._available_connections[0]._reader
+        assert pipe.connection
+        assert pipe.connection in pool._in_use_connections
+        assert not pipe.connection._reader
 
     @pytest.mark.onlynoncluster
     @skip_if_server_version_lt("2.8.8")
@@ -656,23 +754,27 @@ class TestConnection:
             await r.execute_command("DEBUG", "ERROR", "OOM blah blah")
 
     def test_connect_from_url_tcp(self):
-        connection = redis.Redis.from_url("redis://localhost")
+        connection = redis.Redis.from_url("redis://localhost:6379?db=0")
         pool = connection.connection_pool
 
-        assert re.match("(.*)<(.*)<(.*)>>", repr(pool)).groups() == (
+        assert re.match(
+            r"< .*?([^\.]+) \( < .*?([^\.]+) \( (.+) \) > \) >", repr(pool), re.VERBOSE
+        ).groups() == (
             "ConnectionPool",
             "Connection",
-            "host=localhost,port=6379,db=0",
+            "db=0,host=localhost,port=6379",
         )
 
     def test_connect_from_url_unix(self):
         connection = redis.Redis.from_url("unix:///path/to/socket")
         pool = connection.connection_pool
 
-        assert re.match("(.*)<(.*)<(.*)>>", repr(pool)).groups() == (
+        assert re.match(
+            r"< .*?([^\.]+) \( < .*?([^\.]+) \( (.+) \) > \) >", repr(pool), re.VERBOSE
+        ).groups() == (
             "ConnectionPool",
             "UnixDomainSocketConnection",
-            "path=/path/to/socket,db=0",
+            "path=/path/to/socket",
         )
 
     @skip_if_redis_enterprise()
@@ -757,7 +859,7 @@ class TestHealthCheck:
 
     async def test_health_check_in_pipeline(self, r):
         async with r.pipeline(transaction=False) as pipe:
-            pipe.connection = await pipe.connection_pool.get_connection("_")
+            pipe.connection = await pipe.connection_pool.get_connection()
             pipe.connection.next_health_check = 0
             with mock.patch.object(
                 pipe.connection, "send_command", wraps=pipe.connection.send_command
@@ -768,7 +870,7 @@ class TestHealthCheck:
 
     async def test_health_check_in_transaction(self, r):
         async with r.pipeline(transaction=True) as pipe:
-            pipe.connection = await pipe.connection_pool.get_connection("_")
+            pipe.connection = await pipe.connection_pool.get_connection()
             pipe.connection.next_health_check = 0
             with mock.patch.object(
                 pipe.connection, "send_command", wraps=pipe.connection.send_command
@@ -780,7 +882,7 @@ class TestHealthCheck:
     async def test_health_check_in_watched_pipeline(self, r):
         await r.set("foo", "bar")
         async with r.pipeline(transaction=False) as pipe:
-            pipe.connection = await pipe.connection_pool.get_connection("_")
+            pipe.connection = await pipe.connection_pool.get_connection()
             pipe.connection.next_health_check = 0
             with mock.patch.object(
                 pipe.connection, "send_command", wraps=pipe.connection.send_command
@@ -804,7 +906,7 @@ class TestHealthCheck:
     async def test_health_check_in_pubsub_before_subscribe(self, r):
         """A health check happens before the first [p]subscribe"""
         p = r.pubsub()
-        p.connection = await p.connection_pool.get_connection("_")
+        p.connection = await p.connection_pool.get_connection()
         p.connection.next_health_check = 0
         with mock.patch.object(
             p.connection, "send_command", wraps=p.connection.send_command
@@ -826,7 +928,7 @@ class TestHealthCheck:
         connection health
         """
         p = r.pubsub()
-        p.connection = await p.connection_pool.get_connection("_")
+        p.connection = await p.connection_pool.get_connection()
         p.connection.next_health_check = 0
         with mock.patch.object(
             p.connection, "send_command", wraps=p.connection.send_command
@@ -866,7 +968,7 @@ class TestHealthCheck:
         check the connection's health.
         """
         p = r.pubsub()
-        p.connection = await p.connection_pool.get_connection("_")
+        p.connection = await p.connection_pool.get_connection()
         with mock.patch.object(
             p.connection, "send_command", wraps=p.connection.send_command
         ) as m:
